@@ -1,58 +1,19 @@
 """Helpers to check build environment before actual build of scikit-learn"""
 
-import os
-import sys
 import glob
+import os
+import subprocess
+import sys
 import tempfile
 import textwrap
-import setuptools  # noqa
-import subprocess
 
-from distutils.dist import Distribution
-from distutils.sysconfig import customize_compiler
-from numpy.distutils.ccompiler import new_compiler
-from numpy.distutils.command.config_compiler import config_cc
+from setuptools.command.build_ext import customize_compiler, new_compiler
 
 
-def _get_compiler():
-    """Get a compiler equivalent to the one that will be used to build sklearn
-
-    Handles compiler specified as follows:
-        - python setup.py build_ext --compiler=<compiler>
-        - CC=<compiler> python setup.py build_ext
-    """
-    dist = Distribution(
-        {
-            "script_name": os.path.basename(sys.argv[0]),
-            "script_args": sys.argv[1:],
-            "cmdclass": {"config_cc": config_cc},
-        }
-    )
-    dist.parse_config_files()
-    dist.parse_command_line()
-
-    cmd_opts = dist.command_options.get("build_ext")
-    if cmd_opts is not None and "compiler" in cmd_opts:
-        compiler = cmd_opts["compiler"][1]
-    else:
-        compiler = None
-
-    ccompiler = new_compiler(compiler=compiler)
-    customize_compiler(ccompiler)
-
-    return ccompiler
-
-
-def compile_test_program(code, extra_preargs=[], extra_postargs=[]):
+def compile_test_program(code, extra_preargs=None, extra_postargs=None):
     """Check that some C code can be compiled and run"""
-    ccompiler = _get_compiler()
-
-    # extra_(pre/post)args can be a callable to make it possible to get its
-    # value from the compiler
-    if callable(extra_preargs):
-        extra_preargs = extra_preargs(ccompiler)
-    if callable(extra_postargs):
-        extra_postargs = extra_postargs(ccompiler)
+    ccompiler = new_compiler()
+    customize_compiler(ccompiler)
 
     start_dir = os.path.abspath(".")
 
@@ -103,12 +64,10 @@ def basic_check_build():
         # The following check won't work in pyodide
         return
 
-    code = textwrap.dedent(
-        """\
+    code = textwrap.dedent("""\
         #include <stdio.h>
         int main(void) {
         return 0;
         }
-        """
-    )
+        """)
     compile_test_program(code)
