@@ -9,23 +9,24 @@ https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.da
 """
 
 import errno
-from gzip import GzipFile
 import logging
 import os
+from gzip import GzipFile
 from os.path import exists, join
 
-import numpy as np
 import joblib
+import numpy as np
 
-from ._base import _fetch_remote
-from ._base import _convert_data_dataframe
-from . import get_data_home
-from ._base import RemoteFileMetadata
-from ._base import load_descr
-from ..utils import Bunch
-from ..utils import check_random_state
+from ..utils import Bunch, check_random_state
 from ..utils import shuffle as shuffle_method
-
+from ..utils._param_validation import StrOptions, validate_params
+from . import get_data_home
+from ._base import (
+    RemoteFileMetadata,
+    _convert_data_dataframe,
+    _fetch_remote,
+    load_descr,
+)
 
 # The original data can be found at:
 # https://archive.ics.uci.edu/ml/machine-learning-databases/kddcup99-mld/kddcup.data.gz
@@ -46,6 +47,19 @@ ARCHIVE_10_PERCENT = RemoteFileMetadata(
 logger = logging.getLogger(__name__)
 
 
+@validate_params(
+    {
+        "subset": [StrOptions({"SA", "SF", "http", "smtp"}), None],
+        "data_home": [str, os.PathLike, None],
+        "shuffle": ["boolean"],
+        "random_state": ["random_state"],
+        "percent10": ["boolean"],
+        "download_if_missing": ["boolean"],
+        "return_X_y": ["boolean"],
+        "as_frame": ["boolean"],
+    },
+    prefer_skip_nested_validation=True,
+)
 def fetch_kddcup99(
     *,
     subset=None,
@@ -78,9 +92,10 @@ def fetch_kddcup99(
         To return the corresponding classical subsets of kddcup 99.
         If None, return the entire kddcup 99 dataset.
 
-    data_home : str, default=None
+    data_home : str or path-like, default=None
         Specify another download and cache folder for the datasets. By default
         all scikit-learn data is stored in '~/scikit_learn_data' subfolders.
+
         .. versionadded:: 0.19
 
     shuffle : bool, default=False
@@ -96,7 +111,7 @@ def fetch_kddcup99(
         Whether to load only 10 percent of the data.
 
     download_if_missing : bool, default=True
-        If False, raise a IOError if the data is not locally available
+        If False, raise an OSError if the data is not locally available
         instead of trying to download the data from the source site.
 
     return_X_y : bool, default=False
@@ -133,6 +148,10 @@ def fetch_kddcup99(
             The names of the target columns
 
     (data, target) : tuple if ``return_X_y`` is True
+        A tuple of two ndarray. The first containing a 2D array of
+        shape (n_samples, n_features) with each row representing one
+        sample and each column representing the features. The second
+        ndarray of shape (n_samples,) containing the target samples.
 
         .. versionadded:: 0.20
     """
@@ -225,7 +244,6 @@ def fetch_kddcup99(
 
 
 def _fetch_brute_kddcup99(data_home=None, download_if_missing=True, percent10=True):
-
     """Load the kddcup99 dataset, downloading it if necessary.
 
     Parameters
@@ -235,7 +253,7 @@ def _fetch_brute_kddcup99(data_home=None, download_if_missing=True, percent10=Tr
         all scikit-learn data is stored in '~/scikit_learn_data' subfolders.
 
     download_if_missing : bool, default=True
-        If False, raise a IOError if the data is not locally available
+        If False, raise an OSError if the data is not locally available
         instead of trying to download the data from the source site.
 
     percent10 : bool, default=True
@@ -328,7 +346,7 @@ def _fetch_brute_kddcup99(data_home=None, download_if_missing=True, percent10=Tr
             X = joblib.load(samples_path)
             y = joblib.load(targets_path)
         except Exception as e:
-            raise IOError(
+            raise OSError(
                 "The cache for fetch_kddcup99 is invalid, please delete "
                 f"{str(kddcup_dir)} and run the fetch_kddcup99 again"
             ) from e
@@ -362,7 +380,7 @@ def _fetch_brute_kddcup99(data_home=None, download_if_missing=True, percent10=Tr
         joblib.dump(X, samples_path, compress=0)
         joblib.dump(y, targets_path, compress=0)
     else:
-        raise IOError("Data not found and `download_if_missing` is False")
+        raise OSError("Data not found and `download_if_missing` is False")
 
     return Bunch(
         data=X,
