@@ -1,16 +1,17 @@
 """Test the rcv1 loader, if the data is available,
 or if specifically requested via environment variable
-(e.g. for travis cron job)."""
+(e.g. for CI jobs)."""
 
-import scipy.sparse as sp
-import numpy as np
 from functools import partial
+
+import numpy as np
+import scipy.sparse as sp
+
 from sklearn.datasets.tests.test_common import check_return_X_y
-from sklearn.utils._testing import assert_almost_equal
-from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_almost_equal, assert_array_equal
 
 
-def test_fetch_rcv1(fetch_rcv1_fxt):
+def test_fetch_rcv1(fetch_rcv1_fxt, global_random_seed):
     data1 = fetch_rcv1_fxt(shuffle=False)
     X1, Y1 = data1.data, data1.target
     cat_list, s1 = data1.target_names.tolist(), data1.sample_id
@@ -27,24 +28,29 @@ def test_fetch_rcv1(fetch_rcv1_fxt):
     assert (804414,) == s1.shape
     assert 103 == len(cat_list)
 
+    # test descr
+    assert data1.DESCR.startswith(".. _rcv1_dataset:")
+
     # test ordering of categories
-    first_categories = ['C11', 'C12', 'C13', 'C14', 'C15', 'C151']
+    first_categories = ["C11", "C12", "C13", "C14", "C15", "C151"]
     assert_array_equal(first_categories, cat_list[:6])
 
     # test number of sample for some categories
-    some_categories = ('GMIL', 'E143', 'CCAT')
+    some_categories = ("GMIL", "E143", "CCAT")
     number_non_zero_in_cat = (5, 1206, 381327)
     for num, cat in zip(number_non_zero_in_cat, some_categories):
         j = cat_list.index(cat)
         assert num == Y1[:, j].data.size
 
     # test shuffling and subset
-    data2 = fetch_rcv1_fxt(shuffle=True, subset='train', random_state=77)
+    data2 = fetch_rcv1_fxt(
+        shuffle=True, subset="train", random_state=global_random_seed
+    )
     X2, Y2 = data2.data, data2.target
     s2 = data2.sample_id
 
     # test return_X_y option
-    fetch_func = partial(fetch_rcv1_fxt, shuffle=False, subset='train')
+    fetch_func = partial(fetch_rcv1_fxt, shuffle=False, subset="train")
     check_return_X_y(data2, fetch_func)
 
     # The first 23149 samples are the training samples
