@@ -1,34 +1,36 @@
-import pytest
+import itertools
 import warnings
+from functools import partial
 
 import numpy as np
-from functools import partial
-import itertools
+import pytest
 
+import sklearn
 from sklearn.base import clone
-
-from sklearn.exceptions import ConvergenceWarning
-
-from sklearn.utils import check_array
-
-from sklearn.utils._testing import assert_allclose
-from sklearn.utils._testing import assert_array_almost_equal
-from sklearn.utils._testing import assert_array_equal
-from sklearn.utils._testing import ignore_warnings
-from sklearn.utils._testing import TempMemmap
-
-from sklearn.decomposition import DictionaryLearning
-from sklearn.decomposition import MiniBatchDictionaryLearning
-from sklearn.decomposition import SparseCoder
-from sklearn.decomposition import dict_learning
-from sklearn.decomposition import dict_learning_online
-from sklearn.decomposition import sparse_encode
-from sklearn.utils.estimator_checks import check_transformer_data_not_an_array
-from sklearn.utils.estimator_checks import check_transformer_general
-from sklearn.utils.estimator_checks import check_transformers_unfitted
-
+from sklearn.decomposition import (
+    DictionaryLearning,
+    MiniBatchDictionaryLearning,
+    SparseCoder,
+    dict_learning,
+    dict_learning_online,
+    sparse_encode,
+)
 from sklearn.decomposition._dict_learning import _update_dict
-
+from sklearn.exceptions import ConvergenceWarning
+from sklearn.utils import check_array
+from sklearn.utils._testing import (
+    TempMemmap,
+    assert_allclose,
+    assert_array_almost_equal,
+    assert_array_equal,
+    ignore_warnings,
+)
+from sklearn.utils.estimator_checks import (
+    check_transformer_data_not_an_array,
+    check_transformer_general,
+    check_transformers_unfitted,
+)
+from sklearn.utils.parallel import Parallel
 
 rng_global = np.random.RandomState(0)
 n_samples, n_features = 10, 8
@@ -41,7 +43,7 @@ def test_sparse_encode_shapes_omp():
     for n_components, n_samples in itertools.product([1, 5], [1, 9]):
         X_ = rng.randn(n_samples, n_features)
         dictionary = rng.randn(n_components, n_features)
-        for algorithm, n_jobs in itertools.product(algorithms, [1, 3]):
+        for algorithm, n_jobs in itertools.product(algorithms, [1, 2]):
             code = sparse_encode(X_, dictionary, algorithm=algorithm, n_jobs=n_jobs)
             assert code.shape == (n_samples, n_components)
 
@@ -394,8 +396,8 @@ def test_dict_learning_online_positivity(positive_code, positive_dict):
 def test_dict_learning_online_verbosity():
     # test verbosity for better coverage
     n_components = 5
-    from io import StringIO
     import sys
+    from io import StringIO
 
     old_stdout = sys.stdout
     try:
@@ -582,14 +584,6 @@ def test_sparse_encode_error_default_sparsity():
     assert code.shape == (100, 2)
 
 
-def test_unknown_method():
-    n_components = 12
-    rng = np.random.RandomState(0)
-    V = rng.randn(n_components, n_features)  # random init
-    with pytest.raises(ValueError):
-        sparse_encode(X, V, algorithm="<unknown>")
-
-
 def test_sparse_coder_estimator():
     n_components = 12
     rng = np.random.RandomState(0)
@@ -661,84 +655,6 @@ def test_sparse_coder_n_features_in():
     assert sc.n_features_in_ == d.shape[1]
 
 
-@pytest.mark.parametrize("attr", ["iter_offset_", "inner_stats_", "random_state_"])
-def test_minibatch_dict_learning_deprecated_attributes(attr):
-    # check that we raise a deprecation warning when accessing the deprecated
-    # attributes of MiniBatchDictionaryLearning
-    # FIXME: remove in 1.3
-    depr_msg = (
-        f"The attribute `{attr}` is deprecated in 1.1 and will be removed in 1.3."
-    )
-    est = MiniBatchDictionaryLearning(
-        n_components=2, batch_size=4, max_iter=1, random_state=0
-    )
-    est.fit(X)
-
-    with pytest.warns(FutureWarning, match=depr_msg):
-        getattr(est, attr)
-
-
-def test_minibatch_dict_learning_partial_fit_iter_offset_deprecated():
-    # check the deprecation warning of iter_offset in partial_fit
-    # FIXME: remove in 1.3
-    depr_msg = (
-        "'iter_offset' is deprecated in version 1.1 and will be removed in version 1.3"
-    )
-    est = MiniBatchDictionaryLearning(n_components=2, batch_size=4, random_state=0)
-
-    with pytest.warns(FutureWarning, match=depr_msg):
-        est.partial_fit(X, iter_offset=0)
-
-
-def test_minibatch_dict_learning_n_iter_deprecated():
-    # check the deprecation warning of n_iter
-    # FIXME: remove in 1.3
-    depr_msg = (
-        "'n_iter' is deprecated in version 1.1 and will be removed in version 1.4"
-    )
-    est = MiniBatchDictionaryLearning(
-        n_components=2, batch_size=4, n_iter=5, random_state=0
-    )
-
-    with pytest.warns(FutureWarning, match=depr_msg):
-        est.fit(X)
-
-
-@pytest.mark.parametrize(
-    "arg, val",
-    [
-        ("iter_offset", 0),
-        ("inner_stats", None),
-        ("return_inner_stats", False),
-        ("return_n_iter", False),
-        ("n_iter", 5),
-    ],
-)
-def test_dict_learning_online_deprecated_args(arg, val):
-    # check the deprecation warning for the deprecated args of
-    # dict_learning_online
-    # FIXME: remove in 1.3
-    depr_msg = (
-        f"'{arg}' is deprecated in version 1.1 and will be removed in version 1.3."
-    )
-
-    with pytest.warns(FutureWarning, match=depr_msg):
-        dict_learning_online(
-            X, n_components=2, batch_size=4, random_state=0, **{arg: val}
-        )
-
-
-def test_batch_size_default_value_future_warning():
-    # Check that a FutureWarning is raised if batch_size is left to its default value.
-    # FIXME: remove in 1.3
-    msg = "The default value of batch_size will change"
-    with pytest.warns(FutureWarning, match=msg):
-        dict_learning_online(X, n_components=2, random_state=0)
-
-    with pytest.warns(FutureWarning, match=msg):
-        MiniBatchDictionaryLearning(n_components=2, random_state=0).fit(X)
-
-
 def test_update_dict():
     # Check the dict update in batch mode vs online mode
     # Non-regression test for #4866
@@ -762,15 +678,6 @@ def test_update_dict():
     assert_allclose(newd_batch, newd_online)
 
 
-# FIXME: remove in 1.3
-def test_dict_learning_online_n_iter_deprecated():
-    # Check that an error is raised when a deprecated argument is set when max_iter
-    # is also set.
-    msg = "The following arguments are incompatible with 'max_iter'"
-    with pytest.raises(ValueError, match=msg):
-        dict_learning_online(X, max_iter=10, return_inner_stats=True)
-
-
 @pytest.mark.parametrize(
     "algorithm", ("lasso_lars", "lasso_cd", "lars", "threshold", "omp")
 )
@@ -887,8 +794,8 @@ def test_minibatch_dictionary_learning_dtype_match(
 
     assert dict_learner.components_.dtype == expected_type
     assert dict_learner.transform(X.astype(data_type)).dtype == expected_type
-    assert dict_learner._inner_stats[0].dtype == expected_type
-    assert dict_learner._inner_stats[1].dtype == expected_type
+    assert dict_learner._A.dtype == expected_type
+    assert dict_learner._B.dtype == expected_type
 
 
 @pytest.mark.parametrize("method", ("lars", "cd"))
@@ -988,18 +895,24 @@ def test_dict_learning_online_numerical_consistency(method):
     U_64, V_64 = dict_learning_online(
         X.astype(np.float64),
         n_components=n_components,
+        max_iter=1_000,
         alpha=alpha,
         batch_size=10,
         random_state=0,
         method=method,
+        tol=0.0,
+        max_no_improvement=None,
     )
     U_32, V_32 = dict_learning_online(
         X.astype(np.float32),
         n_components=n_components,
+        max_iter=1_000,
         alpha=alpha,
         batch_size=10,
         random_state=0,
         method=method,
+        tol=0.0,
+        max_no_improvement=None,
     )
 
     # Optimal solution (U*, V*) is not unique.
@@ -1038,12 +951,33 @@ def test_get_feature_names_out(estimator):
     )
 
 
-# TODO(1.4) remove
-def test_minibatch_dictionary_learning_warns_and_ignore_n_iter():
-    """Check that we always raise a warning when `n_iter` is set even if it is
-    ignored if `max_iter` is set.
-    """
-    warn_msg = "'n_iter' is deprecated in version 1.1"
+def test_cd_work_on_joblib_memmapped_data(monkeypatch):
+    monkeypatch.setattr(
+        sklearn.decomposition._dict_learning,
+        "Parallel",
+        partial(Parallel, max_nbytes=100),
+    )
+
+    rng = np.random.RandomState(0)
+    X_train = rng.randn(10, 10)
+
+    dict_learner = DictionaryLearning(
+        n_components=5,
+        random_state=0,
+        n_jobs=2,
+        fit_algorithm="cd",
+        max_iter=50,
+        verbose=True,
+    )
+
+    # This must run and complete without error.
+    dict_learner.fit(X_train)
+
+
+# TODO(1.6): remove in 1.6
+def test_xxx():
+    warn_msg = "`max_iter=None` is deprecated in version 1.4 and will be removed"
     with pytest.warns(FutureWarning, match=warn_msg):
-        model = MiniBatchDictionaryLearning(batch_size=256, n_iter=2, max_iter=2).fit(X)
-    assert model.n_iter_ == 2
+        MiniBatchDictionaryLearning(max_iter=None, random_state=0).fit(X)
+    with pytest.warns(FutureWarning, match=warn_msg):
+        dict_learning_online(X, max_iter=None, random_state=0)
