@@ -8,21 +8,21 @@ A Theil-Sen Estimator for Multiple Linear Regression Model
 
 
 import warnings
-from numbers import Integral, Real
 from itertools import combinations
+from numbers import Integral, Real
 
 import numpy as np
+from joblib import effective_n_jobs
 from scipy import linalg
-from scipy.special import binom
 from scipy.linalg.lapack import get_lapack_funcs
-from joblib import Parallel, effective_n_jobs
+from scipy.special import binom
 
-from ._base import LinearModel
-from ..base import RegressorMixin
+from ..base import RegressorMixin, _fit_context
+from ..exceptions import ConvergenceWarning
 from ..utils import check_random_state
 from ..utils._param_validation import Interval
-from ..utils.fixes import delayed
-from ..exceptions import ConvergenceWarning
+from ..utils.parallel import Parallel, delayed
+from ._base import LinearModel
 
 _EPSILON = np.finfo(np.double).eps
 
@@ -395,6 +395,7 @@ class TheilSenRegressor(RegressorMixin, LinearModel):
 
         return n_subsamples, n_subpopulation
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y):
         """Fit linear model.
 
@@ -410,7 +411,6 @@ class TheilSenRegressor(RegressorMixin, LinearModel):
         self : returns an instance of self.
             Fitted `TheilSenRegressor` estimator.
         """
-        self._validate_params()
         random_state = check_random_state(self.random_state)
         X, y = self._validate_data(X, y, y_numeric=True)
         n_samples, n_features = X.shape
