@@ -11,18 +11,17 @@ This module defines export functions for decision trees.
 #          Li Li <aiki.nogard@gmail.com>
 #          Giuseppe Vettigli <vettigli@gmail.com>
 # License: BSD 3 clause
+from collections.abc import Iterable
 from io import StringIO
 from numbers import Integral
 
 import numpy as np
 
-from ..utils.validation import check_is_fitted
 from ..base import is_classifier
-
-from . import _criterion
-from . import _tree
-from ._reingold_tilford import buchheim, Tree
-from . import DecisionTreeClassifier
+from ..utils._param_validation import HasMethods, Interval, StrOptions, validate_params
+from ..utils.validation import check_array, check_is_fitted
+from . import DecisionTreeClassifier, DecisionTreeRegressor, _criterion, _tree
+from ._reingold_tilford import Tree, buchheim
 
 
 def _color_brew(n):
@@ -75,6 +74,24 @@ class Sentinel:
 SENTINEL = Sentinel()
 
 
+@validate_params(
+    {
+        "decision_tree": [DecisionTreeClassifier, DecisionTreeRegressor],
+        "max_depth": [Interval(Integral, 0, None, closed="left"), None],
+        "feature_names": ["array-like", None],
+        "class_names": ["array-like", "boolean", None],
+        "label": [StrOptions({"all", "root", "none"})],
+        "filled": ["boolean"],
+        "impurity": ["boolean"],
+        "node_ids": ["boolean"],
+        "proportion": ["boolean"],
+        "rounded": ["boolean"],
+        "precision": [Interval(Integral, 0, None, closed="left"), None],
+        "ax": "no_validation",  # delegate validation to matplotlib
+        "fontsize": [Interval(Integral, 0, None, closed="left"), None],
+    },
+    prefer_skip_nested_validation=True,
+)
 def plot_tree(
     decision_tree,
     *,
@@ -113,11 +130,11 @@ def plot_tree(
         The maximum depth of the representation. If None, the tree is fully
         generated.
 
-    feature_names : list of str, default=None
+    feature_names : array-like of str, default=None
         Names of each of the features.
         If None, generic names will be used ("x[0]", "x[1]", ...).
 
-    class_names : list of str or bool, default=None
+    class_names : array-like of str or True, default=None
         Names of each of the target classes in ascending numerical order.
         Only relevant for classification and not supported for multi-output.
         If ``True``, shows a symbolic representation of the class name.
@@ -228,7 +245,7 @@ class _BaseTreeExporter:
             color = list(self.colors["rgb"][np.argmax(value)])
             sorted_values = sorted(value, reverse=True)
             if len(sorted_values) == 1:
-                alpha = 0
+                alpha = 0.0
             else:
                 alpha = (sorted_values[0] - sorted_values[1]) / (1 - sorted_values[1])
         else:
@@ -237,8 +254,6 @@ class _BaseTreeExporter:
             alpha = (value - self.colors["bounds"][0]) / (
                 self.colors["bounds"][1] - self.colors["bounds"][0]
             )
-        # unpack numpy scalars
-        alpha = float(alpha)
         # compute the color as alpha against white
         color = [int(round(alpha * c + (1 - alpha) * 255, 0)) for c in color]
         # Return html color code in #RRGGBB format
@@ -256,10 +271,15 @@ class _BaseTreeExporter:
                 # Find max and min values in leaf nodes for regression
                 self.colors["bounds"] = (np.min(tree.value), np.max(tree.value))
         if tree.n_outputs == 1:
-            node_val = tree.value[node_id][0, :] / tree.weighted_n_node_samples[node_id]
-            if tree.n_classes[0] == 1:
-                # Regression
-                node_val = tree.value[node_id][0, :]
+            node_val = tree.value[node_id][0, :]
+            if (
+                tree.n_classes[0] == 1
+                and isinstance(node_val, Iterable)
+                and self.colors["bounds"] is not None
+            ):
+                # Unpack the float only for the regression tree case.
+                # Classification tree requires an Iterable in `get_color`.
+                node_val = node_val.item()
         else:
             # If multi-output color node by impurity
             node_val = -tree.impurity[node_id]
@@ -328,9 +348,9 @@ class _BaseTreeExporter:
             node_string += str(tree.n_node_samples[node_id]) + characters[4]
 
         # Write node class distribution / regression value
-        if self.proportion and tree.n_classes[0] != 1:
+        if not self.proportion and tree.n_classes[0] != 1:
             # For classification this will show the proportion of samples
-            value = value / tree.weighted_n_node_samples[node_id]
+            value = value * tree.weighted_n_node_samples[node_id]
         if labels:
             node_string += "value = "
         if tree.n_classes[0] == 1:
@@ -398,7 +418,6 @@ class _DOTTreeExporter(_BaseTreeExporter):
         precision=3,
         fontname="helvetica",
     ):
-
         super().__init__(
             max_depth=max_depth,
             feature_names=feature_names,
@@ -423,20 +442,6 @@ class _DOTTreeExporter(_BaseTreeExporter):
         else:
             self.characters = ["#", "[", "]", "<=", "\\n", '"', '"']
 
-        # validate
-        if isinstance(precision, Integral):
-            if precision < 0:
-                raise ValueError(
-                    "'precision' should be greater or equal to 0."
-                    " Got {} instead.".format(precision)
-                )
-        else:
-            raise ValueError(
-                "'precision' should be an integer. Got {} instead.".format(
-                    type(precision)
-                )
-            )
-
         # The depth of each node for plotting with 'leaf' option
         self.ranks = {"leaves": []}
         # The colors to render each node with
@@ -507,7 +512,6 @@ class _DOTTreeExporter(_BaseTreeExporter):
 
         # Add node with description
         if self.max_depth is None or depth <= self.max_depth:
-
             # Collect ranks for 'leaf' option in plot_options
             if left_child == _tree.TREE_LEAF:
                 self.ranks["leaves"].append(str(node_id))
@@ -584,7 +588,6 @@ class _MPLTreeExporter(_BaseTreeExporter):
         precision=3,
         fontsize=None,
     ):
-
         super().__init__(
             max_depth=max_depth,
             feature_names=feature_names,
@@ -599,20 +602,6 @@ class _MPLTreeExporter(_BaseTreeExporter):
         )
         self.fontsize = fontsize
 
-        # validate
-        if isinstance(precision, Integral):
-            if precision < 0:
-                raise ValueError(
-                    "'precision' should be greater or equal to 0."
-                    " Got {} instead.".format(precision)
-                )
-        else:
-            raise ValueError(
-                "'precision' should be an integer. Got {} instead.".format(
-                    type(precision)
-                )
-            )
-
         # The depth of each node for plotting with 'leaf' option
         self.ranks = {"leaves": []}
         # The colors to render each node with
@@ -737,6 +726,27 @@ class _MPLTreeExporter(_BaseTreeExporter):
             ax.annotate("\n  (...)  \n", xy_parent, xy, **kwargs)
 
 
+@validate_params(
+    {
+        "decision_tree": "no_validation",
+        "out_file": [str, None, HasMethods("write")],
+        "max_depth": [Interval(Integral, 0, None, closed="left"), None],
+        "feature_names": ["array-like", None],
+        "class_names": ["array-like", "boolean", None],
+        "label": [StrOptions({"all", "root", "none"})],
+        "filled": ["boolean"],
+        "leaves_parallel": ["boolean"],
+        "impurity": ["boolean"],
+        "node_ids": ["boolean"],
+        "proportion": ["boolean"],
+        "rotate": ["boolean"],
+        "rounded": ["boolean"],
+        "special_characters": ["boolean"],
+        "precision": [Interval(Integral, 0, None, closed="left"), None],
+        "fontname": [str],
+    },
+    prefer_skip_nested_validation=True,
+)
 def export_graphviz(
     decision_tree,
     out_file=None,
@@ -772,8 +782,8 @@ def export_graphviz(
 
     Parameters
     ----------
-    decision_tree : decision tree classifier
-        The decision tree to be exported to GraphViz.
+    decision_tree : object
+        The decision tree estimator to be exported to GraphViz.
 
     out_file : object or str, default=None
         Handle or name of the output file. If ``None``, the result is
@@ -786,11 +796,11 @@ def export_graphviz(
         The maximum depth of the representation. If None, the tree is fully
         generated.
 
-    feature_names : list of str, default=None
-        Names of each of the features.
+    feature_names : array-like of shape (n_features,), default=None
+        An array containing the feature names.
         If None, generic names will be used ("x[0]", "x[1]", ...).
 
-    class_names : list of str or bool, default=None
+    class_names : array-like of shape (n_classes,) or bool, default=None
         Names of each of the target classes in ascending numerical order.
         Only relevant for classification and not supported for multi-output.
         If ``True``, shows a symbolic representation of the class name.
@@ -855,6 +865,14 @@ def export_graphviz(
     >>> tree.export_graphviz(clf)
     'digraph Tree {...
     """
+    if feature_names is not None:
+        feature_names = check_array(
+            feature_names, ensure_2d=False, dtype=None, ensure_min_samples=0
+        )
+    if class_names is not None and not isinstance(class_names, bool):
+        class_names = check_array(
+            class_names, ensure_2d=False, dtype=None, ensure_min_samples=0
+        )
 
     check_is_fitted(decision_tree)
     own_file = False
@@ -919,10 +937,23 @@ def _compute_depth(tree, node):
     return max(depths)
 
 
+@validate_params(
+    {
+        "decision_tree": [DecisionTreeClassifier, DecisionTreeRegressor],
+        "feature_names": ["array-like", None],
+        "class_names": ["array-like", None],
+        "max_depth": [Interval(Integral, 0, None, closed="left"), None],
+        "spacing": [Interval(Integral, 1, None, closed="left"), None],
+        "decimals": [Interval(Integral, 0, None, closed="left"), None],
+        "show_weights": ["boolean"],
+    },
+    prefer_skip_nested_validation=True,
+)
 def export_text(
     decision_tree,
     *,
     feature_names=None,
+    class_names=None,
     max_depth=10,
     spacing=3,
     decimals=2,
@@ -939,10 +970,21 @@ def export_text(
         It can be an instance of
         DecisionTreeClassifier or DecisionTreeRegressor.
 
-    feature_names : list of str, default=None
-        A list of length n_features containing the feature names.
+    feature_names : array-like of shape (n_features,), default=None
+        An array containing the feature names.
         If None generic names will be used ("feature_0", "feature_1", ...).
 
+    class_names : array-like of shape (n_classes,), default=None
+        Names of each of the target classes in ascending numerical order.
+        Only relevant for classification and not supported for multi-output.
+
+        - if `None`, the class names are delegated to `decision_tree.classes_`;
+        - otherwise, `class_names` will be used as class names instead of
+          `decision_tree.classes_`. The length of `class_names` must match
+          the length of `decision_tree.classes_`.
+
+        .. versionadded:: 1.3
+
     max_depth : int, default=10
         Only the first max_depth levels of the tree are exported.
         Truncated branches will be marked with "...".
@@ -983,29 +1025,37 @@ def export_text(
     |   |--- petal width (cm) >  1.75
     |   |   |--- class: 2
     """
+    if feature_names is not None:
+        feature_names = check_array(
+            feature_names, ensure_2d=False, dtype=None, ensure_min_samples=0
+        )
+    if class_names is not None:
+        class_names = check_array(
+            class_names, ensure_2d=False, dtype=None, ensure_min_samples=0
+        )
+
     check_is_fitted(decision_tree)
     tree_ = decision_tree.tree_
     if is_classifier(decision_tree):
-        class_names = decision_tree.classes_
+        if class_names is None:
+            class_names = decision_tree.classes_
+        elif len(class_names) != len(decision_tree.classes_):
+            raise ValueError(
+                "When `class_names` is an array, it should contain as"
+                " many items as `decision_tree.classes_`. Got"
+                f" {len(class_names)} while the tree was fitted with"
+                f" {len(decision_tree.classes_)} classes."
+            )
     right_child_fmt = "{} {} <= {}\n"
     left_child_fmt = "{} {} >  {}\n"
     truncation_fmt = "{} {}\n"
 
-    if max_depth < 0:
-        raise ValueError("max_depth bust be >= 0, given %d" % max_depth)
-
     if feature_names is not None and len(feature_names) != tree_.n_features:
         raise ValueError(
             "feature_names must contain %d elements, got %d"
             % (tree_.n_features, len(feature_names))
         )
 
-    if spacing <= 0:
-        raise ValueError("spacing must be > 0, given %d" % spacing)
-
-    if decimals < 0:
-        raise ValueError("decimals must be >= 0, given %d" % decimals)
-
     if isinstance(decision_tree, DecisionTreeClassifier):
         value_fmt = "{}{} weights: {}\n"
         if not show_weights:
@@ -1013,7 +1063,7 @@ def export_text(
     else:
         value_fmt = "{}{} value: {}\n"
 
-    if feature_names:
+    if feature_names is not None:
         feature_names_ = [
             feature_names[i] if i != _tree.TREE_UNDEFINED else None
             for i in tree_.feature
@@ -1023,14 +1073,20 @@ def export_text(
 
     export_text.report = ""
 
-    def _add_leaf(value, class_name, indent):
+    def _add_leaf(value, weighted_n_node_samples, class_name, indent):
         val = ""
-        is_classification = isinstance(decision_tree, DecisionTreeClassifier)
-        if show_weights or not is_classification:
+        if isinstance(decision_tree, DecisionTreeClassifier):
+            if show_weights:
+                val = [
+                    "{1:.{0}f}, ".format(decimals, v * weighted_n_node_samples)
+                    for v in value
+                ]
+                val = "[" + "".join(val)[:-2] + "]"
+                weighted_n_node_samples
+            val += " class: " + str(class_name)
+        else:
             val = ["{1:.{0}f}, ".format(decimals, v) for v in value]
             val = "[" + "".join(val)[:-2] + "]"
-        if is_classification:
-            val += " class: " + str(class_name)
         export_text.report += value_fmt.format(indent, "", val)
 
     def print_tree_recurse(node, depth):
@@ -1047,6 +1103,8 @@ def export_text(
         if tree_.n_classes[0] != 1 and tree_.n_outputs == 1:
             class_name = class_names[class_name]
 
+        weighted_n_node_samples = tree_.weighted_n_node_samples[node]
+
         if depth <= max_depth + 1:
             info_fmt = ""
             info_fmt_left = info_fmt
@@ -1064,11 +1122,11 @@ def export_text(
                 export_text.report += info_fmt_right
                 print_tree_recurse(tree_.children_right[node], depth + 1)
             else:  # leaf
-                _add_leaf(value, class_name, indent)
+                _add_leaf(value, weighted_n_node_samples, class_name, indent)
         else:
             subtree_depth = _compute_depth(tree_, node)
             if subtree_depth == 1:
-                _add_leaf(value, class_name, indent)
+                _add_leaf(value, weighted_n_node_samples, class_name, indent)
             else:
                 trunc_report = "truncated branch of depth %d" % subtree_depth
                 export_text.report += truncation_fmt.format(indent, trunc_report)
