@@ -1,18 +1,24 @@
 """
 Testing for export functions of decision trees (sklearn.tree.export).
 """
+from io import StringIO
 from re import finditer, search
 from textwrap import dedent
 
-from numpy.random import RandomState
+import numpy as np
 import pytest
+from numpy.random import RandomState
 
 from sklearn.base import is_classifier
-from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
 from sklearn.ensemble import GradientBoostingClassifier
-from sklearn.tree import export_graphviz, plot_tree, export_text
-from io import StringIO
 from sklearn.exceptions import NotFittedError
+from sklearn.tree import (
+    DecisionTreeClassifier,
+    DecisionTreeRegressor,
+    export_graphviz,
+    export_text,
+    plot_tree,
+)
 
 # toy sample
 X = [[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]]
@@ -48,48 +54,6 @@ def test_graphviz_toy():
 
     assert contents1 == contents2
 
-    # Test with feature_names
-    contents1 = export_graphviz(
-        clf, feature_names=["feature0", "feature1"], out_file=None
-    )
-    contents2 = (
-        "digraph Tree {\n"
-        'node [shape=box, fontname="helvetica"] ;\n'
-        'edge [fontname="helvetica"] ;\n'
-        '0 [label="feature0 <= 0.0\\ngini = 0.5\\nsamples = 6\\n'
-        'value = [3, 3]"] ;\n'
-        '1 [label="gini = 0.0\\nsamples = 3\\nvalue = [3, 0]"] ;\n'
-        "0 -> 1 [labeldistance=2.5, labelangle=45, "
-        'headlabel="True"] ;\n'
-        '2 [label="gini = 0.0\\nsamples = 3\\nvalue = [0, 3]"] ;\n'
-        "0 -> 2 [labeldistance=2.5, labelangle=-45, "
-        'headlabel="False"] ;\n'
-        "}"
-    )
-
-    assert contents1 == contents2
-
-    # Test with class_names
-    contents1 = export_graphviz(clf, class_names=["yes", "no"], out_file=None)
-    contents2 = (
-        "digraph Tree {\n"
-        'node [shape=box, fontname="helvetica"] ;\n'
-        'edge [fontname="helvetica"] ;\n'
-        '0 [label="x[0] <= 0.0\\ngini = 0.5\\nsamples = 6\\n'
-        'value = [3, 3]\\nclass = yes"] ;\n'
-        '1 [label="gini = 0.0\\nsamples = 3\\nvalue = [3, 0]\\n'
-        'class = yes"] ;\n'
-        "0 -> 1 [labeldistance=2.5, labelangle=45, "
-        'headlabel="True"] ;\n'
-        '2 [label="gini = 0.0\\nsamples = 3\\nvalue = [0, 3]\\n'
-        'class = no"] ;\n'
-        "0 -> 2 [labeldistance=2.5, labelangle=-45, "
-        'headlabel="False"] ;\n'
-        "}"
-    )
-
-    assert contents1 == contents2
-
     # Test plot_options
     contents1 = export_graphviz(
         clf,
@@ -249,6 +213,60 @@ def test_graphviz_toy():
     )
 
 
+@pytest.mark.parametrize("constructor", [list, np.array])
+def test_graphviz_feature_class_names_array_support(constructor):
+    # Check that export_graphviz treats feature names
+    # and class names correctly and supports arrays
+    clf = DecisionTreeClassifier(
+        max_depth=3, min_samples_split=2, criterion="gini", random_state=2
+    )
+    clf.fit(X, y)
+
+    # Test with feature_names
+    contents1 = export_graphviz(
+        clf, feature_names=constructor(["feature0", "feature1"]), out_file=None
+    )
+    contents2 = (
+        "digraph Tree {\n"
+        'node [shape=box, fontname="helvetica"] ;\n'
+        'edge [fontname="helvetica"] ;\n'
+        '0 [label="feature0 <= 0.0\\ngini = 0.5\\nsamples = 6\\n'
+        'value = [3, 3]"] ;\n'
+        '1 [label="gini = 0.0\\nsamples = 3\\nvalue = [3, 0]"] ;\n'
+        "0 -> 1 [labeldistance=2.5, labelangle=45, "
+        'headlabel="True"] ;\n'
+        '2 [label="gini = 0.0\\nsamples = 3\\nvalue = [0, 3]"] ;\n'
+        "0 -> 2 [labeldistance=2.5, labelangle=-45, "
+        'headlabel="False"] ;\n'
+        "}"
+    )
+
+    assert contents1 == contents2
+
+    # Test with class_names
+    contents1 = export_graphviz(
+        clf, class_names=constructor(["yes", "no"]), out_file=None
+    )
+    contents2 = (
+        "digraph Tree {\n"
+        'node [shape=box, fontname="helvetica"] ;\n'
+        'edge [fontname="helvetica"] ;\n'
+        '0 [label="x[0] <= 0.0\\ngini = 0.5\\nsamples = 6\\n'
+        'value = [3, 3]\\nclass = yes"] ;\n'
+        '1 [label="gini = 0.0\\nsamples = 3\\nvalue = [3, 0]\\n'
+        'class = yes"] ;\n'
+        "0 -> 1 [labeldistance=2.5, labelangle=45, "
+        'headlabel="True"] ;\n'
+        '2 [label="gini = 0.0\\nsamples = 3\\nvalue = [0, 3]\\n'
+        'class = no"] ;\n'
+        "0 -> 2 [labeldistance=2.5, labelangle=-45, "
+        'headlabel="False"] ;\n'
+        "}"
+    )
+
+    assert contents1 == contents2
+
+
 def test_graphviz_errors():
     # Check for errors of export_graphviz
     clf = DecisionTreeClassifier(max_depth=3, min_samples_split=2)
@@ -280,13 +298,6 @@ def test_graphviz_errors():
     with pytest.raises(IndexError):
         export_graphviz(clf, out, class_names=[])
 
-    # Check precision error
-    out = StringIO()
-    with pytest.raises(ValueError, match="should be greater or equal"):
-        export_graphviz(clf, out, precision=-1)
-    with pytest.raises(ValueError, match="should be an integer"):
-        export_graphviz(clf, out, precision="1")
-
 
 def test_friedman_mse_in_graphviz():
     clf = DecisionTreeRegressor(criterion="friedman_mse", random_state=0)
@@ -304,7 +315,6 @@ def test_friedman_mse_in_graphviz():
 
 
 def test_precision():
-
     rng_reg = RandomState(2)
     rng_clf = RandomState(8)
     for X, y, clf in zip(
@@ -317,7 +327,6 @@ def test_precision():
             DecisionTreeClassifier(max_depth=1, random_state=0),
         ),
     ):
-
         clf.fit(X, y)
         for precision in (4, 3):
             dot_data = export_graphviz(
@@ -350,33 +359,28 @@ def test_precision():
 def test_export_text_errors():
     clf = DecisionTreeClassifier(max_depth=2, random_state=0)
     clf.fit(X, y)
-
-    err_msg = "max_depth bust be >= 0, given -1"
-    with pytest.raises(ValueError, match=err_msg):
-        export_text(clf, max_depth=-1)
     err_msg = "feature_names must contain 2 elements, got 1"
     with pytest.raises(ValueError, match=err_msg):
         export_text(clf, feature_names=["a"])
-    err_msg = "decimals must be >= 0, given -1"
+    err_msg = (
+        "When `class_names` is an array, it should contain as"
+        " many items as `decision_tree.classes_`. Got 1 while"
+        " the tree was fitted with 2 classes."
+    )
     with pytest.raises(ValueError, match=err_msg):
-        export_text(clf, decimals=-1)
-    err_msg = "spacing must be > 0, given 0"
-    with pytest.raises(ValueError, match=err_msg):
-        export_text(clf, spacing=0)
+        export_text(clf, class_names=["a"])
 
 
 def test_export_text():
     clf = DecisionTreeClassifier(max_depth=2, random_state=0)
     clf.fit(X, y)
 
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |--- feature_1 <= 0.00
     |   |--- class: -1
     |--- feature_1 >  0.00
     |   |--- class: 1
-    """
-    ).lstrip()
+    """).lstrip()
 
     assert export_text(clf) == expected_report
     # testing that leaves at level 1 are not truncated
@@ -384,48 +388,32 @@ def test_export_text():
     # testing that the rest of the tree is truncated
     assert export_text(clf, max_depth=10) == expected_report
 
-    expected_report = dedent(
-        """
-    |--- b <= 0.00
-    |   |--- class: -1
-    |--- b >  0.00
-    |   |--- class: 1
-    """
-    ).lstrip()
-    assert export_text(clf, feature_names=["a", "b"]) == expected_report
-
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |--- feature_1 <= 0.00
     |   |--- weights: [3.00, 0.00] class: -1
     |--- feature_1 >  0.00
     |   |--- weights: [0.00, 3.00] class: 1
-    """
-    ).lstrip()
+    """).lstrip()
     assert export_text(clf, show_weights=True) == expected_report
 
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |- feature_1 <= 0.00
     | |- class: -1
     |- feature_1 >  0.00
     | |- class: 1
-    """
-    ).lstrip()
+    """).lstrip()
     assert export_text(clf, spacing=1) == expected_report
 
     X_l = [[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1], [-1, 1]]
     y_l = [-1, -1, -1, 1, 1, 1, 2]
     clf = DecisionTreeClassifier(max_depth=4, random_state=0)
     clf.fit(X_l, y_l)
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |--- feature_1 <= 0.00
     |   |--- class: -1
     |--- feature_1 >  0.00
     |   |--- truncated branch of depth 2
-    """
-    ).lstrip()
+    """).lstrip()
     assert export_text(clf, max_depth=0) == expected_report
 
     X_mo = [[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]]
@@ -434,14 +422,12 @@ def test_export_text():
     reg = DecisionTreeRegressor(max_depth=2, random_state=0)
     reg.fit(X_mo, y_mo)
 
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |--- feature_1 <= 0.0
     |   |--- value: [-1.0, -1.0]
     |--- feature_1 >  0.0
     |   |--- value: [1.0, 1.0]
-    """
-    ).lstrip()
+    """).lstrip()
     assert export_text(reg, decimals=1) == expected_report
     assert export_text(reg, decimals=1, show_weights=True) == expected_report
 
@@ -449,14 +435,12 @@ def test_export_text():
     reg = DecisionTreeRegressor(max_depth=2, random_state=0)
     reg.fit(X_single, y_mo)
 
-    expected_report = dedent(
-        """
+    expected_report = dedent("""
     |--- first <= 0.0
     |   |--- value: [-1.0, -1.0]
     |--- first >  0.0
     |   |--- value: [1.0, 1.0]
-    """
-    ).lstrip()
+    """).lstrip()
     assert export_text(reg, decimals=1, feature_names=["first"]) == expected_report
     assert (
         export_text(reg, decimals=1, show_weights=True, feature_names=["first"])
@@ -464,6 +448,30 @@ def test_export_text():
     )
 
 
+@pytest.mark.parametrize("constructor", [list, np.array])
+def test_export_text_feature_class_names_array_support(constructor):
+    # Check that export_graphviz treats feature names
+    # and class names correctly and supports arrays
+    clf = DecisionTreeClassifier(max_depth=2, random_state=0)
+    clf.fit(X, y)
+
+    expected_report = dedent("""
+    |--- b <= 0.00
+    |   |--- class: -1
+    |--- b >  0.00
+    |   |--- class: 1
+    """).lstrip()
+    assert export_text(clf, feature_names=constructor(["a", "b"])) == expected_report
+
+    expected_report = dedent("""
+    |--- feature_1 <= 0.00
+    |   |--- class: cat
+    |--- feature_1 >  0.00
+    |   |--- class: dog
+    """).lstrip()
+    assert export_text(clf, class_names=constructor(["cat", "dog"])) == expected_report
+
+
 def test_plot_tree_entropy(pyplot):
     # mostly smoke tests
     # Check correctness of export_graphviz for criterion = entropy
@@ -505,7 +513,6 @@ def test_plot_tree_gini(pyplot):
 
 
 def test_not_fitted_tree(pyplot):
-
     # Testing if not fitted tree throws the correct error
     clf = DecisionTreeRegressor()
     with pytest.raises(NotFittedError):
